/// @file Model.h
/// @description
/// @author learnopengl.com - Updated by Josh
/// @lastedit 2024-02-22
/// @license Unlicense - Public Domain

#pragma once

#include <cstdlib>
#include <glad/glad.h>
#include <assimp/Importer.hpp>
#include <assimp/scene.h>
#include <assimp/postprocess.h>
#include <string>
#include <vector>
#include <LearnOpenGL/Mesh.h>
#include <LearnOpenGL/Shader.h>

namespace LearnOpenGL
{
    uint TextureFromFile(const char *path, const std::string& directory, bool gamma = false);


    class Model
    {
    public:
        // model data
        std::vector<LearnOpenGL::Texture2D> textures_loaded; // Stores all the textures loaded so far, optimization to make sure textures aren't loaded more than once
        std::vector<LearnOpenGL::Mesh> meshes;
        std::string directory;
        bool gammaCorrection;

        // Expects a filepath to a 3D model
        Model(const std::string& path, bool gamma = false) : gammaCorrection(gamma)
        {
            loadModel(path);
        }

        // Draws the model, and this all its meshes
        void Draw(Shader& shader)
        {
            for(auto& mesh : meshes)
                mesh.Draw(shader);
        }
    protected:
    private:
        // loads a model with supported ASSIMP extensions from file and stores the resulting meshes in the meshes vector
        void loadModel(const std::string& path)
        {
            // read file via ASSIMP
            Assimp::Importer importer;
        }
    };
}